/*----------------------------------------------------------------------
  File    : jim.h
  Contents: jim algorithm for finding frequent item sets
  Author  : Christian Borgelt
  History : 2014.08.25 file created
            2014.08.29 functions jim_data() and jim_report() added
----------------------------------------------------------------------*/
#ifndef __JIM__
#define __JIM__
#include "report.h"

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
/* --- target pattern types --- */
#define JIM_FREQ      ISR_FREQUENT  /* frequent item sets */
#define JIM_FREQUENT  ISR_FREQUENT  /* frequent item sets */
#define JIM_CLOSED    ISR_CLOSED    /* closed  frequent item sets */
#define JIM_MAXIMAL   ISR_MAXIMAL   /* maximal frequent item sets */

/* --- evaluation measures --- */
#define JIM_LDRATIO   1         /* binary log. of support quotient */
#define JIM_SIZESIM   2         /* item set size times similarity */

/* --- similarity measures --- */
#define JIM_NONE             0  /* always 0 */
#define JIM_RUSSEL_RAO       1  /* S_R = s/n */
#define JIM_KULCZYNSKI       2  /* S_K = s/q */
#define JIM_JACCARD          3  /* S_J = s/r */
#define JIM_TANIMOTO         3
#define JIM_DICE             4  /* S_D = 2s/(r+s) */
#define JIM_SORENSEN         4
#define JIM_CZEKANOWSKI      4
#define JIM_SOKAL_SNEATH_1   5  /* S_S = s/(r+q) */
#define JIM_SOKAL_MICHENER   6  /* S_M = (s+z)/n */
#define JIM_HAMMING          6
#define JIM_FAITH            7  /* S_F = (s+z/2)/n */
#define JIM_ROGERS_TANIMOTO  8  /* S_T = (s+z)/(n+q) */
#define JIM_SOKAL_SNEATH_2   9  /* S_N = 2(s+z)/(n+s+z) */
#define JIM_GOWER_LEGENDRE   9
#define JIM_SOKAL_SNEATH_3  10  /* S_O = (s+z)/q */
#define JIM_BARONI_BUSER    11  /* S_B = (x+s)/(x+r) */
#define JIM_GENERIC         12  /* S   = (c_0s +c_1z +c_2n +c_3x)
                                       / (c_4s +c_5z +c_6n +c_7x) */

/* n = total number of transactions                          */
/* s = number of transactions with all items in the item set */
/* r = number of transactions with at least one item in set  */
/* q = number of transactions with some (but not all) items  */
/* z = number of transactions with no items in the set       */
/* x = \sqrt{sz} (geometric mean of s and z)                 */

/* --- algorithm variants --- */
#define JIM_BASIC     0         /* basic variant */
#define JIM_AUTO      0         /* automatic algorithm choice */

/* --- operation modes --- */
#define JIM_PERFECT   0x0010    /* prune with perfect extensions */
#define JIM_PREFMT    0x1000    /* pre-format integer numbers */
#define JIM_AP4CM     0x2000    /* aggressive closed/maximal pruning */
#ifdef USE_ZLIB                 /* if optional output compression */
#define JIM_ZLIB      0x4000    /* flag for output compression */
#endif
#define JIM_DEFAULT   JIM_PERFECT
#ifdef NDEBUG
#define JIM_NOCLEAN   0x8000    /* do not clean up memory */
#else                           /* in function jim() */
#define JIM_NOCLEAN   0         /* in debug version */
#endif                          /* always clean up memory */
#define JIM_VERBOSE   INT_MIN   /* verbose message output */

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef struct _jim             /* Jaccard item set miner */
JIM;                            /* (opaque structure) */

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/
extern JIM* jim_create (int target, double smin, ITEM zmin, ITEM zmax,
                        int eval, double thresh,
                        int covsim, double *simps, double sim,
                        int algo, int mode);
extern void jim_delete (JIM *jim, int deldar);
extern int  jim_data   (JIM *jim, TABAG *tabag, int sort);
extern int  jim_report (JIM *jim, ISREPORT *report);
extern int  jim_mine   (JIM *jim);
#endif
